@@ -3,13 +3,14 @@
  *
  * Used in conjunction with the libxlsxwriter library.
  *
- * Copyright 2014-2019, John McNamara, jmcnamara@cpan.org. See LICENSE.txt.
+ * Copyright 2014-2021, John McNamara, jmcnamara@cpan.org. See LICENSE.txt.
  *
  */
 
 #include "xlsxwriter/xmlwriter.h"
 #include "xlsxwriter/common.h"
 #include "xlsxwriter/drawing.h"
+#include "xlsxwriter/worksheet.h"
 #include "xlsxwriter/utility.h"
 
 #define LXW_OBJ_NAME_LENGTH 14  /* "Picture 65536", or "Chart 65536" */
@@ -54,7 +55,6 @@ lxw_free_drawing_object(lxw_drawing_object *drawing_object)
         return;
 
     free(drawing_object->description);
-    free(drawing_object->url);
     free(drawing_object->tip);
 
     free(drawing_object);
@@ -221,6 +221,107 @@ _drawing_write_to(lxw_drawing *self, lxw_drawing_coords *coords)
     lxw_xml_end_tag(self->file, "xdr:to");
 }
 
+/*
+ * Write the <a:hlinkClick> element.
+ */
+STATIC void
+_drawing_write_a_hlink_click(lxw_drawing *self, uint32_t rel_index, char *tip)
+{
+    struct xml_attribute_list attributes;
+    struct xml_attribute *attribute;
+    char xmlns_r[] = "http://schemas.openxmlformats.org/"
+        "officeDocument/2006/relationships";
+    char r_id[LXW_MAX_ATTRIBUTE_LENGTH];
+
+    lxw_snprintf(r_id, LXW_ATTR_32, "rId%d", rel_index);
+
+    LXW_INIT_ATTRIBUTES();
+    LXW_PUSH_ATTRIBUTES_STR("xmlns:r", xmlns_r);
+    LXW_PUSH_ATTRIBUTES_STR("r:id", r_id);
+
+    if (tip)
+        LXW_PUSH_ATTRIBUTES_STR("tooltip", tip);
+
+    lxw_xml_empty_tag(self->file, "a:hlinkClick", &attributes);
+
+    LXW_FREE_ATTRIBUTES();
+}
+
+/*
+ * Write the <a16:creationId> element.
+ */
+STATIC void
+_drawing_write_a16_creation_id(lxw_drawing *self)
+{
+    struct xml_attribute_list attributes;
+    struct xml_attribute *attribute;
+    char xmlns[] = "http://schemas.microsoft.com/office/drawing/2014/main";
+
+    LXW_INIT_ATTRIBUTES();
+    LXW_PUSH_ATTRIBUTES_STR("xmlns:a16", xmlns);
+    LXW_PUSH_ATTRIBUTES_STR("id", "{00000000-0008-0000-0000-000002000000}");
+
+    lxw_xml_empty_tag(self->file, "a16:creationId", &attributes);
+
+    LXW_FREE_ATTRIBUTES();
+}
+
+/*
+ * Write the <adec:decorative> element.
+ */
+STATIC void
+_workbook_write_adec_decorative(lxw_drawing *self)
+{
+    struct xml_attribute_list attributes;
+    struct xml_attribute *attribute;
+    char xmlns[] =
+        "http://schemas.microsoft.com/office/drawing/2017/decorative";
+
+    LXW_INIT_ATTRIBUTES();
+    LXW_PUSH_ATTRIBUTES_STR("xmlns:adec", xmlns);
+    LXW_PUSH_ATTRIBUTES_STR("val", "1");
+
+    lxw_xml_empty_tag(self->file, "adec:decorative", &attributes);
+
+    LXW_FREE_ATTRIBUTES();
+}
+
+/*
+ * Write the <a:ext> element.
+ */
+STATIC void
+_drawing_write_uri_ext(lxw_drawing *self, char *uri)
+{
+    struct xml_attribute_list attributes;
+    struct xml_attribute *attribute;
+
+    LXW_INIT_ATTRIBUTES();
+    LXW_PUSH_ATTRIBUTES_STR("uri", uri);
+
+    lxw_xml_start_tag(self->file, "a:ext", &attributes);
+
+    LXW_FREE_ATTRIBUTES();
+}
+
+/*
+ * Write the decorative elements.
+ */
+STATIC void
+_workbook_write_decorative(lxw_drawing *self)
+{
+    lxw_xml_start_tag(self->file, "a:extLst", NULL);
+
+    _drawing_write_uri_ext(self, "{FF2B5EF4-FFF2-40B4-BE49-F238E27FC236}");
+    _drawing_write_a16_creation_id(self);
+    lxw_xml_end_tag(self->file, "a:ext");
+
+    _drawing_write_uri_ext(self, "{C183D7F6-B498-43B3-948B-1728B52AA6E4}");
+    _workbook_write_adec_decorative(self);
+    lxw_xml_end_tag(self->file, "a:ext");
+
+    lxw_xml_end_tag(self->file, "a:extLst");
+}
+
 /*
  * Write the <xdr:cNvPr> element.
  */
@@ -240,11 +341,32 @@ _drawing_write_c_nv_pr(lxw_drawing *self, char *object_name, uint32_t index,
     LXW_PUSH_ATTRIBUTES_STR("name", name);
 
     if (drawing_object && drawing_object->description
-        && strlen(drawing_object->description)) {
+        && strlen(drawing_object->description)
+        && !drawing_object->decorative) {
+
         LXW_PUSH_ATTRIBUTES_STR("descr", drawing_object->description);
     }
 
-    lxw_xml_empty_tag(self->file, "xdr:cNvPr", &attributes);
+    if (drawing_object
+        && (drawing_object->url_rel_index || drawing_object->decorative)) {
+        lxw_xml_start_tag(self->file, "xdr:cNvPr", &attributes);
+
+        if (drawing_object->url_rel_index) {
+            /* Write the a:hlinkClick element. */
+            _drawing_write_a_hlink_click(self,
+                                         drawing_object->url_rel_index,
+                                         drawing_object->tip);
+        }
+
+        if (drawing_object->decorative) {
+            _workbook_write_decorative(self);
+        }
+
+        lxw_xml_end_tag(self->file, "xdr:cNvPr");
+    }
+    else {
+        lxw_xml_empty_tag(self->file, "xdr:cNvPr", &attributes);
+    }
 
     LXW_FREE_ATTRIBUTES();
 }
@@ -474,7 +596,7 @@ _drawing_write_pic(lxw_drawing *self, uint32_t index,
     _drawing_write_nv_pic_pr(self, index, drawing_object);
 
     /* Write the xdr:blipFill element. */
-    _drawing_write_blip_fill(self, index);
+    _drawing_write_blip_fill(self, drawing_object->rel_index);
 
     /* Write the xdr:spPr element. */
     _drawing_write_sp_pr(self, drawing_object);
@@ -663,7 +785,8 @@ _drawing_write_a_graphic(lxw_drawing *self, uint32_t index)
  * Write the <xdr:graphicFrame> element.
  */
 STATIC void
-_drawing_write_graphic_frame(lxw_drawing *self, uint32_t index)
+_drawing_write_graphic_frame(lxw_drawing *self, uint32_t index,
+                             uint32_t rel_index)
 {
     struct xml_attribute_list attributes;
     struct xml_attribute *attribute;
@@ -680,7 +803,7 @@ _drawing_write_graphic_frame(lxw_drawing *self, uint32_t index)
     _drawing_write_xfrm(self);
 
     /* Write the a:graphic element. */
-    _drawing_write_a_graphic(self, index);
+    _drawing_write_a_graphic(self, rel_index);
 
     lxw_xml_end_tag(self->file, "xdr:graphicFrame");
 
@@ -699,24 +822,21 @@ _drawing_write_two_cell_anchor(lxw_drawing *self, uint32_t index,
 
     LXW_INIT_ATTRIBUTES();
 
-    if (drawing_object->anchor_type == LXW_ANCHOR_TYPE_IMAGE) {
-
-        if (drawing_object->edit_as == LXW_ANCHOR_EDIT_AS_ABSOLUTE)
-            LXW_PUSH_ATTRIBUTES_STR("editAs", "absolute");
-        else if (drawing_object->edit_as != LXW_ANCHOR_EDIT_AS_RELATIVE)
-            LXW_PUSH_ATTRIBUTES_STR("editAs", "oneCell");
-    }
+    if (drawing_object->anchor == LXW_OBJECT_MOVE_DONT_SIZE)
+        LXW_PUSH_ATTRIBUTES_STR("editAs", "oneCell");
+    else if (drawing_object->anchor == LXW_OBJECT_DONT_MOVE_DONT_SIZE)
+        LXW_PUSH_ATTRIBUTES_STR("editAs", "absolute");
 
     lxw_xml_start_tag(self->file, "xdr:twoCellAnchor", &attributes);
 
     _drawing_write_from(self, &drawing_object->from);
     _drawing_write_to(self, &drawing_object->to);
 
-    if (drawing_object->anchor_type == LXW_ANCHOR_TYPE_CHART) {
+    if (drawing_object->type == LXW_DRAWING_CHART) {
         /* Write the xdr:graphicFrame element for charts. */
-        _drawing_write_graphic_frame(self, index);
+        _drawing_write_graphic_frame(self, index, drawing_object->rel_index);
     }
-    else if (drawing_object->anchor_type == LXW_ANCHOR_TYPE_IMAGE) {
+    else if (drawing_object->type == LXW_DRAWING_IMAGE) {
         /* Write the xdr:pic element. */
         _drawing_write_pic(self, index, drawing_object);
     }
@@ -774,7 +894,7 @@ _drawing_write_pos(lxw_drawing *self, int32_t x, int32_t y)
  * Write the <xdr:absoluteAnchor> element.
  */
 STATIC void
-_drawing_write_absolute_anchor(lxw_drawing *self)
+_drawing_write_absolute_anchor(lxw_drawing *self, uint32_t frame_index)
 {
     lxw_xml_start_tag(self->file, "xdr:absoluteAnchor", NULL);
 
@@ -793,7 +913,7 @@ _drawing_write_absolute_anchor(lxw_drawing *self)
         _drawing_write_ext(self, 6162675, 6124575);
     }
 
-    _drawing_write_graphic_frame(self, 1);
+    _drawing_write_graphic_frame(self, frame_index, frame_index);
 
     /* Write the xdr:clientData element. */
     _drawing_write_client_data(self);
@@ -832,7 +952,7 @@ lxw_drawing_assemble_xml_file(lxw_drawing *self)
     }
     else {
         /* Write the xdr:absoluteAnchor element. Mainly for chartsheets. */
-        _drawing_write_absolute_anchor(self);
+        _drawing_write_absolute_anchor(self, 1);
     }
 
     lxw_xml_end_tag(self->file, "xdr:wsDr");
